/********************************************************
* RANDOM TURING MACHINE SIMULATION                      *
* THEORY OF COMPUTER SCIENCE                            *
* PROF.: Joaquim Quinteiro Ucha [joukim@comp.ufla.br]  *
*                                                       *
* DEVELOPERS:                                           *
* Anderson de Rezende Rocha [undersun@comp.ufla.br]     *
* Jlio Csar Alves         [jcalves@comp.ufla.br]      *
*                                                       *
* BOTH computer scientists undergraduating students   at  *
* UFLA, Lavras Federal University at Lavras, MG, BRAZIL *
********************************************************/

// This file implements a simple application using Random Turing Machine and a RTM's macros' idea
// The user chooses one or other

#include <iostream>
#include "randomTuring.h"
#include "macros.h"
#include <vector>
#include "machineOutput.h"

// TMachine will be the randomTuring class or rtMacros class
template<class TMachine>
class aplication
{
    public:
        aplication();
        ~aplication() {}

        // gets a word from user
        vector<char> getInput();
        // shows the tape
        void showVector(vector<char> aVector);
        // shows the machine's status
        void showStatus(int i);
        // load a machine from a file
        void loadMachine();
        // changes the machine's transictions' limit
        void changeLimit();
        // proccess a word with a required transictions' number
        void parser();
        // continues the proccessing of a word
        void proccess();
        // shows the menu
        void menu();
        // "Thank you for playing"
        void exit();

    private:
        // the machine
        TMachine machine;
        // the machine was already loaded?
        bool machineLoaded;
        // the word was already took?
        bool inputSet;
        // the user desires to see the transcitions?
        bool desiredPrint;
        // machine's transictions' limit
        int limit;
};

template<class TMachine>
aplication<TMachine>::aplication()
{
    // initializing the bools
    machineLoaded = false;
    inputSet = false;
    desiredPrint = false;

    // showing the menu
    menu();
}

// gets a word from user
template<class TMachine>
vector<char> aplication<TMachine>::getInput()
{
    string input;
    vector<char> word;

    cout << "Type the input word: " << endl;
    cin >> input;

    for ( int i = 0; i < input.size(); i++)
        word.push_back(input[i]);

    return word;
}

// shows the tape
template<class TMachine>
void aplication<TMachine>::showVector(vector<char> aVector)
{
    cout << "<";
    for (int i = 0; i < aVector.size(); i++)
    {
        cout << aVector[i];
    }
    cout << ">" << endl;
}


// shows the machine's status
template<class TMachine>
void aplication<TMachine>::showStatus(int i)
{
    switch(i)
    {
        case 0 :
            cout << "Machine not started";
            break;
        case 1 :
            cout << "Word acepted";
            break;
        case 2 :
            cout << "Word rejected";
            break;
        case 3 :
            cout << "Machine is still proccessing";
            break;
        case 4 :
            cout << "Machine's transictions limit arrived";
            break;
        case 5 :
            cout << "There's a invalid symbol at word";
            break;
        default:
            cout << "Error: Invalid flag";
    }
}

// changes the machine's transictions' limit
template<class TMachine>
void aplication<TMachine>::changeLimit()
{
    do
    {
        cout << "Enter the machine's transictions limit: ";
        cin >> limit;
        if (limit < 0)
            cout << "The machine's transictions limit must be non-negative\n" << endl;
    } while(limit < 0);
    machine.setTLimit(limit);
}

// load a machine from a file
template<class TMachine>
void aplication<TMachine>::loadMachine()
{
    string fileName;

    cout << "Enter a file's name: ";
    cin >> fileName;
    if (machine.readFile(fileName) == 0)
    {
        cout << "Error: Machine didn't load" << endl;
        machineLoaded = false;
    }
    else
    {
        cout << "Machine '" << machine.getDescription() << "' loaded" << endl;
        machineLoaded = true;
        changeLimit();
    }
    inputSet = false;
}

// proccess a word with a required transictions' number
template<class TMachine>
void aplication<TMachine>::parser()
{
    machine.setTape(getInput());
    inputSet = true;
    proccess();
}

// continues the proccessing of a word
template<class TMachine>
void aplication<TMachine>::proccess()
{
    int nTransictions;
    char garbage[2];

    do
    {
        cout << "Enter the number of transictions (0 -> unlimited): ";
        cin >> nTransictions;
        if (nTransictions < 0)
            cout << "The number of transictions must be non-negative\n" << endl;
    } while (nTransictions < 0);
    cin.getline(garbage, sizeof(garbage));
    machine.proccess(nTransictions, desiredPrint);
    showVector(machine.getTape());
    showStatus(static_cast<int>(machine.getStatus()));
    cout << endl;
}


// "That's all folks (R)"
template<class TMachine>
void aplication<TMachine>::exit()
{
    cout << "\n\nThank you for playing\n" << endl;
}

// show the menu
template<class TMachine>
void aplication<TMachine>::menu()
{
    string option;

    do
    {
        cout << "\n\n\tRANDOM TURING MACHINE\n\t\tby Anderson de Rezende Rocha and Julio Cesar Alves\n" << endl;
        cout << "1 - Load a machine\n"
             << "2 - Enter a input word to proccess\n"
             << "3 - Proccess more transictions\n"
             << "4 - Proccess step by step\n"
             << "5 - Change machine's transictions limit\n";
        if (!desiredPrint)
            cout << "6 - Select to show the transictions\n";
        else
            cout << "6 - Unselect to show the transictions\n";
        cout << "7 - Exit\n" << endl;
        cout << "Option: ";
        cin >> option;
        switch (option[0])
        {
            case '1' :
                loadMachine();
                break;
            case '2' :
                if (!machineLoaded)
                {
                    cout << "You must still to load a machine" << endl;
                    loadMachine();
                }
                if (machineLoaded)
                    parser();
                break;
            case '3' :
                if (!machineLoaded)
                {
                    cout << "You must still to load a machine" << endl;
                    loadMachine();
                }
                if (machineLoaded)
                {
                    if (!inputSet)
                    {
                        cout << "You must still to enter a input word" << endl;
                        parser();
                    }
                    else
                        proccess();
                }
                break;
            case '4' :
                if (machineLoaded && inputSet)
                {
                    machine.proccess(1, true);
                    showStatus(static_cast<int>(machine.getStatus()));
                }

                else
                    cout << "You must stil to enter a machine or a word" << endl;
                break;
            case '5' :
                if (machineLoaded)
                    changeLimit();
                else
                    cout << "No machine loaded" << endl;
                break;
            case '6' :
                desiredPrint = !desiredPrint;
                if (desiredPrint)
                    cout << "Transictions WILL BE printed" << endl;
                else
                    cout << "Transictions WILL NOT BE printed " << endl;
                break;
            case '7' :
                exit();
                break;
            default :
                cout << "Invalid option" << endl;
        }
    } while(option[0] != '7');
}

int main(void)
{
    string option;

    do
    {
        // choosing the machine's type
        cout << "\nDo you preffer work with:\n1 - Simple Random Turing Machine" << endl;
        cout << "2 - Random Turing Machine's Macros" << endl;
        cout << "Option: ";
        cin >> option;
        switch (option[0])
        {
            case '1' :
                aplication<randomTuring<char, machineOutput> > *appRTM;
                appRTM = new aplication<randomTuring<char, machineOutput> >;
                delete appRTM;
                break;
            case '2' :
                aplication<rtMacros<machineOutput> > *appMacros;
                appMacros = new aplication<rtMacros<machineOutput> >;
                delete appMacros;
                break;
            default :
                cout << "Invalid Option" << endl;
        }
    } while (option[0] != '1' && option[0] != '2');
    exit(0);
}

