/********************************************************
* RANDOM TURING MACHINE SIMULATION                      *
* THEORY OF COMPUTER SCIENCE                            *
* PROF.: Joaquim Quinteiro Ucha [joukim@comp.ufla.br]  *
*                                                       *
* DEVELOPERS:                                           *
* Anderson de Rezende Rocha [undersun@comp.ufla.br]     *
* Jlio Csar Alves         [jcalves@comp.ufla.br]      *
*                                                       *
* BOTH computer scientists undergraduating students   at  *
* UFLA, Lavras Federal University at Lavras, MG, BRAZIL *
********************************************************/

// Implementation of a LIST. NodeList is used to constructed a 
// linked nodes list.

using namespace std;

#ifndef LIST_H
#define LIST_H

#include "nodeList.h"

template <class TValue>
class list
{
    public:
        list();
        list(const list<TValue> &list);
        ~list();
        // Erase all list's elements
        void list<TValue>::eraseAll();
        // Returns the list's head
        nodeList<TValue> *getHead() const;
        // Insert a element at the end of the list
        void push_back(const TValue &val);
        // Find one element at the list and refresh it
        bool find(TValue& valor);
      bool isEmpty() { return (getHead() == NULL); }
    protected:
        // Store the list head element
        nodeList<TValue> *head;
};

template <class TValue>
list<TValue>::list()
{
    head = NULL;
}

template <class TValue>
list<TValue>::list(const list<TValue> &list)
{
    nodeList<TValue> *node;

    // Creates a empty list...
    head = NULL;
    node = list.head();

    // and copies each element
    while (node != NULL)
    {
        insertBack(node->getValue());
        node = node->getNext();
    }
}

template <class TValue>
list<TValue>::~list()
{
    eraseAll(); //remover todos os nohs
}

// Erase all elements of the List making it empty
template <class TValue>
void list<TValue>::eraseAll()
{
    nodeList<TValue> *aux;

    // destroying all nodes...
    aux = head;
    while(aux != NULL)
    {
        head = aux->getNext();
        delete aux;
        aux = head;
    }
}

// Returns the head list element
template <class TValue>
nodeList<TValue> *list<TValue>::getHead() const
{
    return head;
}

// Inserts a element at the end of the list
template <class TValue>
void list<TValue>::push_back(const TValue &val)
// NOTE: inserts a copy of the passed value
{
    nodeList<TValue> *actual;
    nodeList<TValue> *past;

    actual = head;
    past = NULL;

    while (actual != NULL)
    {
        past = actual;
        actual = actual->getNext();
    }
    actual = new nodeList<TValue>;
    actual->copyValue(val);
    if (past == NULL)
        head = actual;
    else
        past->setNext(actual);
}

// Find one element at the list and refresh its
// Returns FALSE if element is not present otherwise returns TRUE
template <class TValue>
bool list<TValue>::find(TValue &val)
{
    if (head != NULL)
    {
        nodeList<TValue> *auxNode;
        auxNode = head;
        while (auxNode != NULL)
        {
            if (val == auxNode->getValue())
            {
                val = auxNode->getValue();
                return true;
            }
            else
                auxNode = auxNode->getNext();
        }
    }
    return false;
}

#endif

