/********************************************************
* RANDOM TURING MACHINE SIMULATION                      *
* THEORY OF COMPUTER SCIENCE                            *
* PROF.: Joaquim Quinteiro Ucha [joukim@comp.ufla.br]  *
*                                                       *
* DEVELOPERS:                                           *
* Anderson de Rezende Rocha [undersun@comp.ufla.br]     *
* Jlio Csar Alves         [jcalves@comp.ufla.br]      *
*                                                       *
* BOTH computer scientists undergraduating students   at  *
* UFLA, Lavras Federal University at Lavras, MG, BRAZIL *
********************************************************/

// class machineOutput prints a Random Turing Machine's transictions on default output

#include <iostream>
#include "randomTuring.h"

#ifndef MACHINEOUTPUT_H
#define MACHINEOUTPUT_H

using namespace std;

class machineOutput
{
    public :
        machineOutput() {}
        ~machineOutput() {}
        // print a transiction
        void print(vector<char> tape, int stateId, int cursor,
                   transiction<char> transiction, regTuring registers);

    private :
        // print a Random Turing Machine's register's function
        void printFunction(regFunction function, int reg1, int reg2);
};

// print a transiction
void machineOutput::print(vector<char> tape, int stateId, int cursor,
                          transiction<char> transiction, regTuring registers)
{
    // printing the tape
    cout << "<";
    for (int i = 0; i < tape.size(); i++)
    {
        cout << tape[i];
    }
    cout << "> ";

    // printing the register's function
    printFunction(transiction.getRegFunction(), transiction.getReg1(), transiction.getReg2());

    // printing the register's value
    cout << " [ ";
    for (int i = 0; i < registers.nRegisters(); i++)
    {
        if (i == transiction.getRegForMove())
            cout << "*";
        cout << registers.getValue(i) << " ";
    }
    cout << "]" << endl;

    // printing the cursor that show the tape's head's position
    for(int i = 0; i < registers.getValue(transiction.getRegForMove())+1; i++)
        cout << " ";
    cout << "^" << endl;

    // printing the state's id
    for(int i = 0; i < registers.getValue(transiction.getRegForMove())+1; i++)
        cout << " ";
    cout << "q" << stateId << endl;
    cout << endl;
}

// print a Random Turing Machine's register's function
void machineOutput::printFunction(regFunction function, int reg1, int reg2)
{
    cout << "(";
    switch (function)
    {
        case NOTHING:
            cout << "NOTHING";
            break;
        case INC:
            cout << "INC " << reg1;
            break;
        case DEC:
            cout << "DEC " << reg1;
            break;
        case ADD:
            cout << "ADD " << reg1 << " " << reg2;
            break;
        case SUB:
            cout << "SUB " << reg1 << " " << reg2;
            break;
        case ZEROES:
            cout << "ZEROES " << reg1;
            break;
        case COPY:
            cout << "COPY " << reg1 << " " << reg2;
            break;
        case ISZERO:
            cout << "ISZERO " << reg1 << " " << reg2;
            break;
        case ISEQUAL:
            cout << "ISEQUAL " << reg1 << " " << reg2;
            break;
        case GT:
            cout << "GT " << reg1 << " " << reg2;
            break;
        default:
            cout << "";
    }
    cout << ")";
}

#endif



