/********************************************************
* RANDOM TURING MACHINE SIMULATION                      *
* THEORY OF COMPUTER SCIENCE                            *
* PROF.: Joaquim Quinteiro Ucha [joukim@comp.ufla.br]  *
*                                                       *
* DEVELOPERS:                                           *
* Anderson de Rezende Rocha [undersun@comp.ufla.br]     *
* Jlio Csar Alves         [jcalves@comp.ufla.br]      *
*                                                       *
* BOTH computer scientists undergraduating students   at  *
* UFLA, Lavras Federal University at Lavras, MG, BRAZIL *
********************************************************/

// Defines a Random Turing Machine's STATE

#ifndef STATE_H
#define STATE_H

#include "list.h"
#include "regTuring.h"
using namespace std;

// Creating a link to transiction class. This is a compiler directive.
template<class TSymbol>
class transiction;

// Definition of state class
template<class TSymbol>
class state
{
    public:
        state<TSymbol>();
        // Alternative constructor. This is more usual.
        state<TSymbol>(int stateId, bool isFinal);
        state<TSymbol>(const state<TSymbol>& state);
        ~state<TSymbol>();
        // Returns the state ID
        int getId();
        // Returns if this state is final or not
        bool isFinalState();
        // Inserts a transition from this state to a target state
        bool insertTransiction(TSymbol readSymbol, state<TSymbol>* targetState, TSymbol writeSymbol,
                                regFunction function, int reg1 = 0, int reg2 = 0, int regForMove = 0);
        // Process a tape symbol
        transiction<TSymbol> *proccess(TSymbol *readSymbol);
        // Sets the next state. Used to internal representation of the random turing machine
        void setNextState(state<TSymbol> *state);
        // Returns the next state. Used to internal representation
        state<TSymbol> *getNextState();
        // Returns the state's transiction
        list<transiction<TSymbol> > *getTransictions() const;
        // Returns TRUE id the state has not anything transiction
        bool hasNotTransictions() { return getTransictions()->isEmpty(); }
        // print a state on default output
        void print();

    private:
        // Stores the state id
        int id;
        // Stores the information to identifies if this state is final or not
        bool isFinal;
        // Stores a reference to a next state. For internal use only
        state<TSymbol> *nextState;
        // List of the transictions of this state
        list<transiction<TSymbol> > *transictions;
};

template<class TSymbol>
state<TSymbol>::state<TSymbol>()
{
    id = 0;
    isFinal = false;
    nextState = NULL;
    transictions = new list<transiction<TSymbol> >();
}

// Alternative constructor. This is more usual.
template<class TSymbol>
state<TSymbol>::state<TSymbol>(int stateId, bool isFinal)
{
    id = stateId;
    this->isFinal = isFinal;
    nextState = NULL;
    transictions = new list<transiction<TSymbol> >();
}

template<class TSymbol>
state<TSymbol>::state<TSymbol>(const state<TSymbol>& state)
{
    id = state.id;
    isFinal = state.isFinal;
    transictions = state.transictions;
    nextState = state.nextState;
}

template<class TSymbol>
state<TSymbol>::~state<TSymbol>()
{
    delete transictions;
}

// Returns the state ID
template<class TSymbol>
int state<TSymbol>::getId()
{
    return id;
}

// Returns if this state is a final state or not
template<class TSymbol>
bool state<TSymbol>::isFinalState()
{
    return isFinal;
}

// Inserts a transition from this state to a target state
template<class TSymbol>
bool state<TSymbol>::insertTransiction(TSymbol readSymbol, state<TSymbol>* targetState, TSymbol writeSymbol,
                                               regFunction function, int reg1, int reg2, int regForMove)
{
    transiction<TSymbol> *aTransiction;
    aTransiction = new transiction<TSymbol>(readSymbol, targetState, writeSymbol, function, reg1, reg2,
                                            regForMove);
    transictions->push_back(*aTransiction);
    return true;
}

// Process a tape symbol
template<class TSymbol>
transiction<TSymbol> *state<TSymbol>::proccess(TSymbol *readSymbol)
{
    transiction<TSymbol > *aTransiction;

    aTransiction = new transiction<TSymbol>(*readSymbol);
    if(!transictions->find(*aTransiction))
    {
        delete aTransiction;
        return (NULL);
    }
    else
        return  (aTransiction);
}

// Sets the next state. Used to internal representation of the turing machine
template<class TSymbol>
void state<TSymbol>::setNextState(state<TSymbol> *state)
{
    nextState = state;
}

// Returns the next state. Used to internal representation
template<class TSymbol>
state<TSymbol> *state<TSymbol>::getNextState()
{
    return nextState;
}

// Returns the state's transiction
template<class TSymbol>
list<transiction<TSymbol> > *state<TSymbol>::getTransictions() const
{
    return transictions;
}

// print a transiction on default output
template<class TSymbol>
void state<TSymbol>::print()
{
    nodeList<transiction<TSymbol> > *node;
    node = new nodeList<transiction<TSymbol> >();

    if (isFinalState())
        cout << "*";
    cout << "q" << getId() << " ";

    node = transictions->getHead();

    transiction<TSymbol> *aTransiction;
    aTransiction = new transiction<TSymbol>();

    while (node != NULL)
    {
        (*aTransiction) = node->getValue();
        cout << "<";
        aTransiction->print();
        cout << "> ";
        node = node->getNext();
    }
    delete aTransiction;
}

#endif


