/********************************************************
* RANDOM TURING MACHINE SIMULATION                      *
* THEORY OF COMPUTER SCIENCE                            *
* PROF.: Joaquim Quinteiro Ucha [joukim@comp.ufla.br]  *
*                                                       *
* DEVELOPERS:                                           *
* Anderson de Rezende Rocha [undersun@comp.ufla.br]     *
* Jlio Csar Alves         [jcalves@comp.ufla.br]      *
*                                                       *
* BOTH computer scientists undergraduating students   at  *
* UFLA, Lavras Federal University at Lavras, MG, BRAZIL *
********************************************************/

// Defines a Randon Turing Machine's TRANSICTION to represent a turing machine as a DYNAMICAL GRAPH

#ifndef TRANSICTION_H
#define TRANSICTION_H

#include "state.h"
#include "regTuring.h"

using namespace std;

// Creating a link to state class. This is a compiler directive.
template<class TSymbol>
class state;

// Definition of transiction class
template<class TSymbol>
class transiction
{
    public:
        transiction<TSymbol>();
        transiction<TSymbol>(TSymbol readSymbol);
        // Alternative constructor. More used.
        transiction<TSymbol>(TSymbol readSymbol, state<TSymbol>* targetState, TSymbol writeSymbol,
                                               regFunction function, int reg1=0, int reg2=0, int regForMove=0);
        transiction<TSymbol>(const transiction<TSymbol> &aTransiction);
        ~transiction<TSymbol>() {};

        // Gets the read symbol
        TSymbol getReadSymbol();
        // Gets the write symbol
        TSymbol getWriteSymbol();
        // Gets the target state.
        state<TSymbol> *getTargetState();
        // Gets the register's function
        regFunction getRegFunction();
        // Gets the first register
        int getReg1();
        // Gets the second register
        int getReg2();
        // Gets the register used to move the tape's head
        int getRegForMove();

        // Sets the read symbol
        TSymbol setReadSymbol(TSymbol &symbol);
        // Sets the write symbol
        TSymbol setWriteSymbol(TSymbol &symbol);
        // Sets the tarset state.
        state<TSymbol> *setTargetState(state<TSymbol>* aState);
        // Sets the register's function
        regFunction setRegFunction(regFunction aFunction);
        // Sets the first register
        int setReg1(int reg);
        // Sets the second register
        int setReg2(int reg);
        // Sets the register used to move the tape's head
        int setRegForMove(int reg);

        // Overloadind the == operator to compares two transictions by the read symbol
        bool operator == (transiction<TSymbol> aTransiction);
        // print a transiction on default output
        void print();

    private:
        // Stores a pointer to the targetState. Internal use.
        state<TSymbol> *targetState;
        // Stores the read symbol
        TSymbol readSymbol;
        // Stores the write symbol
        TSymbol writeSymbol;
        // Stores the register's function
        regFunction function;
        // Stores the first register
        int reg1;
        // Stores the second register
        int reg2;
        // Stores the register used to move the tape's head
        int regForMove;

};

template <class TSymbol>
transiction<TSymbol>::transiction<TSymbol>()
{
    this->targetState = NULL;
}

// Default constructor
template <class TSymbol>
transiction<TSymbol>::transiction<TSymbol>(TSymbol readSymbol)
{
    this->readSymbol = readSymbol;
    this->targetState = NULL;
}

// Alternative constructor. More used.
template <class TSymbol>
transiction<TSymbol>::transiction<TSymbol>(TSymbol readSymbol, state<TSymbol>* targetState, TSymbol writeSymbol,
                                               regFunction function, int reg1, int reg2, int regForMove)
{
    this->readSymbol = readSymbol;
    this->writeSymbol = writeSymbol;
    this->targetState = targetState;
    this->function = function;
    this->reg1 = reg1;
    this->reg2 = reg2;
    this->regForMove = regForMove;
}

// Copy constructor
template <class TSymbol>
transiction<TSymbol>::transiction<TSymbol>(const transiction<TSymbol> &aTransiction)
{
    readSymbol = aTransiction.readSymbol;
    writeSymbol = aTransiction.writeSymbol;
    targetState = aTransiction.targetState;
    function = aTransiction.function;
    reg1 = aTransiction.reg1;
    reg2 = aTransiction.reg2;
    regForMove = aTransiction.regForMove;
}

// Gets a read symbol
template <class TSymbol>
TSymbol transiction<TSymbol>::getReadSymbol()
{
    return readSymbol;
}

// Gets a write symbol
template <class TSymbol>
TSymbol transiction<TSymbol>::getWriteSymbol()
{
    return writeSymbol;
}

template <class TSymbol>
state<TSymbol> *transiction<TSymbol>::getTargetState()
{
    return targetState;
}

// Gets a register function id
template <class TSymbol>
regFunction transiction<TSymbol>::getRegFunction()
{
    return function;
}

template <class TSymbol>
int transiction<TSymbol>::getReg1()
{
    return reg1;
}

template <class TSymbol>
int transiction<TSymbol>::getReg2()
{
    return reg2;
}

// Gets a register id.
template <class TSymbol>
int transiction<TSymbol>::getRegForMove()
{
    return regForMove;
}

// Sets the read symbol
template <class TSymbol>
TSymbol setReadSymbol(TSymbol &symbol)
{
    readSymbol = symbol;
}

// Sets the write symbol
template <class TSymbol>
TSymbol setWriteSymbol(TSymbol &symbol)
{
    writeSymbol = symbol;
}

// Sets the tarset state
template <class TSymbol>
state<TSymbol> *setTargetState(state<TSymbol>* aState)
{
    targetState = aState;
}

// Sets the register's function
template <class TSymbol>
regFunction setRegFunction(regFunction aFunction)
{
    function = aFunction;
}

// Sets the first register
template <class TSymbol>
int setReg1(int reg)
{
    reg1 = reg;
}

// Sets the second register
template <class TSymbol>
int setReg2(int reg)
{
    reg2 = reg;
}

// Sets the register used to move the tape's head
template <class TSymbol>
int setRegForMove(int reg)
{
    regForMove = reg;
}

// Overloadind the == operator to compares two transictions by the read symbol
template <class TSymbol>
bool transiction<TSymbol>::operator == (transiction<TSymbol> aTransiction)
{
    return (getReadSymbol() == aTransiction.getReadSymbol());
}

// print a transiction on default output
template <class TSymbol>
void transiction<TSymbol>::print()
{
    cout << getReadSymbol() << " q" << getTargetState()->getId() << " " << getWriteSymbol() << " "
         << getRegFunction() << " " << getReg1() << " " << getReg2() << " " << getRegForMove();
}

#endif

